
/***************************************************************
File:   2ds.c - 2-Dimensional Stress Equation
Desc:   Given any two orthogonal stesses, x and y and the
        shear stress, xy, the stress transformation is calculated
        to give the maximum and minimum principal stresses.  The
        rotation of the stress element from the +x-axis to 1 or
        2 is also calculated.  CCW is assumed to be positive for
        this angle () and for shear directions.
        The maximum shear stress is correctly calculated from the
        triaxial stress state.

David Edwards <edwards@dee-engineering.com>
***************************************************************/

#include <stdio.h>
#include <math.h>
#include <mathext.h>
#include <conio.h>

void input (char *str, int *addr)
{
        printf(str);
        scanf("%lf", addr);
}

main()
{
    double sx,sy,tauxy,sa,sb,phi,s1,s2,s3,sp,tau1,taumax,save;

    input("\n  Tensile Stress, x [ksi]: ",&sx);
    input("  Tensile Stress, y [ksi]: ",&sy);
    input("  Shear Stress, xy  [ksi]: ",&tauxy);
    rpc(.5*(sx-sy),tauxy,&tau1,&phi);
    save=.5*(sx+sy);
    sa=save+tau1;
    sb=save-tau1;
    if (sa >= 0.0) {
        if (sb >= 0.0) {
            s1=sa;
            s2=sb;
            s3=0.0;
            taumax=fabs(.5*s1);
        }
        else {
            s1=sa;
            s2=0.0;
            s3=sb;
            taumax=fabs(.5*(s1-s3));
        }

        printf("\n  Angle from the +x-axis to 1 (principal direction)"
               " [deg]: %6.1lf\n",phi*90.0/M_PI);
    }
    else {
            s1=0.0;
            s2=sa;
            s3=sb;
            taumax=fabs(.5*s3);
            printf("\n  Angle from the +x-axis to 2 (principal direction)"
                   " [deg]: %6.1lf\n",phi*90.0/M_PI);
    }

    sp=sqrt(sa*sa-sa*sb+sb*sb);

    printf("\n  Maximum principal stress,      1 [ksi]: %10.2lf\n",s1);
    printf("  Intermediate principal stress, 2 [ksi]: %10.2lf\n",s2);
    printf("  Minimum principal stress,      3 [ksi]: %10.2lf\n",s3);
    printf("\n  Maximum shear stress,        max [ksi]: %10.2lf\n",taumax);
    printf("\n  von Mises stress,              ' [ksi]: %10.2lf\n",sp);

}
